<?php

namespace Drupal\calculation_fields\Element;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\Number;
use Drupal\webform\Utility\WebformElementHelper;
use Symfony\Component\ExpressionLanguage\ExpressionLanguage;

/**
 * @FormElement("form_element_math")
 */
class FormMathElement extends Number {

  public function getInfo() {
    $info = parent::getInfo();
    $info['#evaluation_fields'] = NULL;
    $info['#process'] = [self::class . '::processFormMathElement'];
    $info['#element_validate'] = [self::class . '::validate'];
    return $info;
  }

  public static function validate(&$element, FormStateInterface $form_state, &$complete_form) {
    $expression = static::mountEvaluation($element["#evaluation_fields"], $form_state);
    $value = NestedArray::getValue($form_state->getValues(), $element['#parents']);
    if (empty($expression) || empty($value)) {
      return;
    }
    $expressionLanguage = new ExpressionLanguage();
    $result = $expressionLanguage->evaluate($expression);
    if ($result !== $value) {
      $form_state->setValue($element["#name"], $result);
    }
  }

  /**
   * {@inheritDoc}
   */
  public static function preRenderNumber($element) {
    $element['#attributes']['type'] = 'number';
    Element::setAttributes($element, ['id', 'name', 'value', 'step', 'min', 'max', 'placeholder', 'size', 'fields']);
    static::setAttributes($element, ['form-number', 'js-calc-field']);
    $element['#attributes']['data-evaluate-expression'] = $element["#evaluation_fields"];
    $element['#attributes']['data-evaluate-fields'] = static::evaluateFields($element["#evaluation_fields"]);
    $element['#attached']['library'][] = 'calculation_fields/calculation_fields';
    return $element;
  }

  public static function processFormMathElement($element, \Drupal\Core\Form\FormStateInterface $form_state, $form) {
    $element['#attributes']['data-evaluate-context'] = str_replace('_', '-', $form["#form_id"]);
    $dependency_fields = static::evaluateFields($element["#evaluation_fields"]);

    $values_dependencies = [];
    foreach ($dependency_fields as $field) {
      if (!empty($form['elements'])) {
        $d_element = WebformElementHelper::getElement($form['elements'], $field);
      }
      else {
        $d_element = WebformElementHelper::getElement($form, $field);
      }

      if (!$d_element['#access']) {
        $values_dependencies[$field] = $form_state->getValue($field);
      }
    }
    $element['#attributes']['data-evaluate-deep-fields'] = json_encode($values_dependencies);
    return $element;
  }

  protected static function mountEvaluation($evaluation, FormStateInterface $form_state) {
    $fields = static::evaluateFields($evaluation);
    $collected_values = [];
    foreach ($fields as $field) {
      $value = $form_state->getValue($field);
      if (!empty($value)) {
        $collected_values[$field] = $value;
      }
    }
    if (count($collected_values) !== count(array_unique($fields))) {
      return null;
    }
    $exp = $evaluation;
    foreach ($collected_values as $field => $value) {
      $exp = str_replace(':' . $field, $value, $exp);
    }
    return $exp;
  }



  public static function evaluateFields($evaluation_fields): array {
    $fields = explode(':', $evaluation_fields) ?? [];
    $fields = array_filter($fields);
    $patterns = [];
    foreach ($fields as $field) {
      $items = explode(' ', $field);
      $items = array_map(function ($item) {
        return preg_replace("/[^a-zA-Z0-9_]/", "", $item);
      }, $items);
      $filteredItems = array_filter($items, function ($item) {
        return !is_numeric($item) && !empty($item);
      });
      $patterns = array_merge($patterns, $filteredItems);
    }
    return $patterns;
  }


}
